@@ -24,8 +24,9 @@
  * @short_description: Asynchronous message bus subsystem
  * @see_also: #GstMessage, #GstElement
  *
- * The #GstBus is an object responsible for delivering #GstMessages in
- * a first-in first-out way from the streaming threads to the application.
+ * The #GstBus is an object responsible for delivering #GstMessage packets in
+ * a first-in first-out way from the streaming threads (see #GstTask) to the
+ * application.
  *
  * Since the application typically only wants to deal with delivery of these
  * messages from one thread, the GstBus will marshall the messages between
@@ -44,12 +45,12 @@
  *
  * The bus can be polled with the gst_bus_poll() method. This methods blocks
  * up to the specified timeout value until one of the specified messages types
- * is posted on the bus. The application can then _pop() the messages from the
- * bus to handle them.
+ * is posted on the bus. The application can then gst_bus_pop() the messages
+ * from the bus to handle them.
  * Alternatively the application can register an asynchronous bus function
  * using gst_bus_add_watch_full() or gst_bus_add_watch(). This function will
- * install a #GSource in the default glib main loop and will deliver messages 
- * a short while after they have been posted. Note that the main loop should 
+ * install a #GSource in the default glib main loop and will deliver messages
+ * a short while after they have been posted. Note that the main loop should
  * be running for the asynchronous callbacks.
  *
  * It is also possible to get messages from the bus without any thread
@@ -87,8 +88,6 @@ enum
   LAST_SIGNAL
 };
 
-static void gst_bus_class_init (GstBusClass * klass);
-static void gst_bus_init (GstBus * bus);
 static void gst_bus_dispose (GObject * object);
 
 static void gst_bus_set_main_context (GstBus * bus, GMainContext * ctx);
@@ -141,7 +140,7 @@ gst_bus_class_init (GstBusClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_bus_dispose);
+  gobject_class->dispose = gst_bus_dispose;
 
   /**
    * GstBus::sync-message:
@@ -273,7 +272,7 @@ gst_bus_new (void)
 {
   GstBus *result;
 
-  result = g_object_new (gst_bus_get_type (), NULL);
+  result = g_object_newv (gst_bus_get_type (), 0, NULL);
   GST_DEBUG_OBJECT (result, "created new bus");
 
   return result;
@@ -402,7 +401,7 @@ is_flushing:
  * Check if there are pending messages on the bus that
  * should be handled.
  *
- * Returns: TRUE if there are messages on the bus to be handled, FALSE 
+ * Returns: TRUE if there are messages on the bus to be handled, FALSE
  * otherwise.
  *
  * MT safe.
@@ -943,7 +942,7 @@ poll_func (GstBus * bus, GstMessage * message, GstBusPollData * poll_data)
   type = GST_MESSAGE_TYPE (message);
 
   if (type & poll_data->events) {
-    g_return_if_fail (poll_data->message == NULL);
+    g_assert (poll_data->message == NULL);
     /* keep ref to message */
     poll_data->message = gst_message_ref (message);
     GST_DEBUG ("mainloop %p quit", poll_data->loop);
@@ -970,7 +969,7 @@ poll_destroy (GstBusPollData * poll_data, gpointer unused)
   poll_data->source_running = FALSE;
   if (!poll_data->timeout_id) {
     g_main_loop_unref (poll_data->loop);
-    g_free (poll_data);
+    g_slice_free (GstBusPollData, poll_data);
   }
 }
 
@@ -980,7 +979,7 @@ poll_destroy_timeout (GstBusPollData * poll_data)
   poll_data->timeout_id = 0;
   if (!poll_data->source_running) {
     g_main_loop_unref (poll_data->loop);
-    g_free (poll_data);
+    g_slice_free (GstBusPollData, poll_data);
   }
 }
 
@@ -1035,7 +1034,7 @@ gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
   GstMessage *ret;
   gulong id;
 
-  poll_data = g_new0 (GstBusPollData, 1);
+  poll_data = g_slice_new (GstBusPollData);
   poll_data->source_running = TRUE;
   poll_data->loop = g_main_loop_new (NULL, FALSE);
   poll_data->events = events;
@@ -1134,7 +1133,7 @@ gst_bus_sync_signal_handler (GstBus * bus, GstMessage * message, gpointer data)
  * Instructs GStreamer to emit the "sync-message" signal after running the bus's
  * sync handler. This function is here so that code can ensure that they can
  * synchronously receive messages without having to affect what the bin's sync
- * handler is. 
+ * handler is.
  *
  * This function may be called multiple times. To clean up, the caller is
  * responsible for calling gst_bus_disable_sync_message_emission() as many times
