@@ -60,6 +60,7 @@
 #include "gst_private.h"
 
 #include "gstelement.h"
+#include "gstelementdetails.h"
 #include "gstinfo.h"
 #include "gsturi.h"
 #include "gstregistry.h"
@@ -69,10 +70,7 @@
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
-static void gst_element_factory_class_init (GstElementFactoryClass * klass);
-static void gst_element_factory_init (GstElementFactory * factory);
 static void gst_element_factory_finalize (GObject * object);
-void __gst_element_details_clear (GstElementDetails * dp);
 static void gst_element_factory_cleanup (GstElementFactory * factory);
 
 static GstPluginFeatureClass *parent_class = NULL;
@@ -99,7 +97,7 @@ gst_element_factory_class_init (GstElementFactoryClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_element_factory_finalize);
+  gobject_class->finalize = gst_element_factory_finalize;
 }
 
 static void
@@ -150,45 +148,6 @@ gst_element_factory_find (const gchar * name)
   return NULL;
 }
 
-void
-__gst_element_details_clear (GstElementDetails * dp)
-{
-  g_free (dp->longname);
-  g_free (dp->klass);
-  g_free (dp->description);
-  g_free (dp->author);
-  memset (dp, 0, sizeof (GstElementDetails));
-}
-
-#define VALIDATE_SET(__dest, __src, __entry)                            \
-G_STMT_START {                                                          \
-  if (g_utf8_validate (__src->__entry, -1, NULL)) {                     \
-    __dest->__entry = g_strdup (__src->__entry);                        \
-  } else {                                                              \
-    g_warning ("Invalid UTF-8 in " G_STRINGIFY (__entry) ": %s",        \
-        __src->__entry);                                                \
-    __dest->__entry = g_strdup ("[ERROR: invalid UTF-8]");              \
-  }                                                                     \
-} G_STMT_END
-
-void
-__gst_element_details_set (GstElementDetails * dest,
-    const GstElementDetails * src)
-{
-  VALIDATE_SET (dest, src, longname);
-  VALIDATE_SET (dest, src, klass);
-  VALIDATE_SET (dest, src, description);
-  VALIDATE_SET (dest, src, author);
-}
-
-void
-__gst_element_details_copy (GstElementDetails * dest,
-    const GstElementDetails * src)
-{
-  __gst_element_details_clear (dest);
-  __gst_element_details_set (dest, src);
-}
-
 static void
 gst_element_factory_cleanup (GstElementFactory * factory)
 {
@@ -218,7 +177,7 @@ gst_element_factory_cleanup (GstElementFactory * factory)
       g_ptr_array_free (caps->structs, TRUE);
       caps->refcount = 0;
     }
-    g_free (templ);
+    g_slice_free (GstStaticPadTemplate, templ);
   }
   g_list_free (factory->staticpadtemplates);
   factory->staticpadtemplates = NULL;
@@ -265,8 +224,8 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   registry = gst_registry_get_default ();
 
   /* check if feature already exists, if it exists there is no need to update it
-   * when the registry is getting updated, outdated plugins and all there
-   * feature are removed and readded.
+   * when the registry is getting updated, outdated plugins and all their
+   * features are removed and readded.
    */
   existing_feature = gst_registry_lookup_feature (registry, name);
   if (existing_feature) {
@@ -281,7 +240,8 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
 
   factory =
-      GST_ELEMENT_FACTORY_CAST (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
+      GST_ELEMENT_FACTORY_CAST (g_object_newv (GST_TYPE_ELEMENT_FACTORY, 0,
+          NULL));
   gst_plugin_feature_set_name (GST_PLUGIN_FEATURE_CAST (factory), name);
   GST_LOG_OBJECT (factory, "Created new elementfactory for type %s",
       g_type_name (type));
@@ -299,10 +259,11 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstPadTemplate *templ = item->data;
     GstStaticPadTemplate *newt;
 
-    newt = g_new0 (GstStaticPadTemplate, 1);
+    newt = g_slice_new (GstStaticPadTemplate);
     newt->name_template = g_intern_string (templ->name_template);
     newt->direction = templ->direction;
     newt->presence = templ->presence;
+    newt->static_caps.caps.refcount = 0;
     newt->static_caps.string = gst_caps_to_string (templ->caps);
     factory->staticpadtemplates =
         g_list_append (factory->staticpadtemplates, newt);
@@ -404,8 +365,14 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
   if (factory->type == 0)
     goto no_type;
 
-  /* create an instance of the element, cast so we don't assert on NULL */
-  element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
+  /* create an instance of the element, cast so we don't assert on NULL
+   * also set name as early as we can
+   */
+  if (name)
+    element =
+        GST_ELEMENT_CAST (g_object_new (factory->type, "name", name, NULL));
+  else
+    element = GST_ELEMENT_CAST (g_object_newv (factory->type, 0, NULL));
   if (G_UNLIKELY (element == NULL))
     goto no_element;
 
@@ -419,9 +386,6 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
           (gpointer) & oclass->elementfactory, NULL, factory))
     gst_object_unref (factory);
 
-  if (name)
-    gst_object_set_name (GST_OBJECT_CAST (element), name);
-
   GST_DEBUG ("created element \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
 
   return element;
