@@ -74,7 +74,7 @@ static
     void
 read_tsc (gint64 * dst)
 {
-#ifdef HAVE_RDTSC
+#if defined(HAVE_RDTSC) && defined(__GNUC__)
   guint64 tsc;
   __asm__ __volatile__ ("rdtsc":"=A" (tsc));
 
@@ -111,9 +111,9 @@ gint _gst_trace_on = 1;
  * Returns: a new #GstTrace.
  */
 GstTrace *
-gst_trace_new (gchar * filename, gint size)
+gst_trace_new (const gchar * filename, gint size)
 {
-  GstTrace *trace = g_malloc (sizeof (GstTrace));
+  GstTrace *trace = g_slice_new (GstTrace);
 
   g_return_val_if_fail (trace != NULL, NULL);
   trace->filename = g_strdup (filename);
@@ -152,7 +152,7 @@ gst_trace_destroy (GstTrace * trace)
     gst_trace_flush (trace);
   close (trace->fd);
   g_free (trace->buf);
-  g_free (trace);
+  g_slice_free (GstTrace, trace);
 }
 
 /**
@@ -285,7 +285,7 @@ _gst_alloc_trace_register (const gchar * name)
 
   g_return_val_if_fail (name, NULL);
 
-  trace = g_new0 (GstAllocTrace, 1);
+  trace = g_slice_new (GstAllocTrace);
   trace->name = g_strdup (name);
   trace->live = 0;
   trace->mem_live = NULL;
@@ -471,7 +471,13 @@ gst_alloc_trace_print (const GstAllocTrace * trace)
     mem_live = trace->mem_live;
 
     while (mem_live) {
-      g_print ("%-22.22s : %p\n", "", mem_live->data);
+      gpointer data = mem_live->data;
+
+      if (G_IS_OBJECT (data)) {
+        g_print ("%-22.22s : %p\n", g_type_name (G_OBJECT_TYPE (data)), data);
+      } else {
+        g_print ("%-22.22s : %p\n", "", data);
+      }
       mem_live = mem_live->next;
     }
   }
