@@ -28,10 +28,6 @@
  * inside the 'check' directories of various GStreamer packages.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "gstcheck.h"
 
 GST_DEBUG_CATEGORY (check_debug);
@@ -86,88 +82,6 @@ static void gst_check_log_critical_func
     _gst_check_raised_warning = TRUE;
 }
 
-#if defined(G_OS_UNIX) && defined(HAVE_EXECINFO_H)
-#include <signal.h>
-#include <execinfo.h>
-#include <unistd.h>
-
-static struct sigaction oldaction_segv;
-static struct sigaction oldaction_ill;
-static struct sigaction oldaction_bus;
-static struct sigaction oldaction_abrt;
-static gboolean _gst_check_fault_handler_is_setup;      /* FALSE */
-
-static void
-_gst_check_fault_handler_restore (void)
-{
-  if (!_gst_check_fault_handler_is_setup)
-    return;
-
-  _gst_check_fault_handler_is_setup = FALSE;
-
-  sigaction (SIGSEGV, &oldaction_segv, NULL);
-  sigaction (SIGILL, &oldaction_ill, NULL);
-  sigaction (SIGBUS, &oldaction_bus, NULL);
-  sigaction (SIGABRT, &oldaction_abrt, NULL);
-}
-
-static void
-_gst_check_fault_handler_sighandler (int signum)
-{
-  void *bt_arr[100];
-  int num;
-  char *signame;
-
-  /* We need to restore the fault handler or we'll keep getting it */
-  _gst_check_fault_handler_restore ();
-
-  switch (signum) {
-    case SIGSEGV:
-      signame = "SIGSEGV";
-      break;
-    case SIGILL:
-      signame = "SIGILL";
-      break;
-    case SIGBUS:
-      signame = "SIGBUS";
-      break;
-    case SIGABRT:
-      signame = "SIGABRT";
-      break;
-    default:
-      signame = "Unknown Signal";
-      break;
-  }
-
-  fprintf (stderr, "\nERROR: Caught signal '%s' while running test.\n",
-      signame);
-
-  fprintf (stderr, "Backtrace:\n");
-
-  if ((num = backtrace ((void **) bt_arr, G_N_ELEMENTS (bt_arr))))
-    backtrace_symbols_fd (bt_arr, num, STDERR_FILENO);
-}
-
-static void
-_gst_check_fault_handler_setup (void)
-{
-  struct sigaction action;
-
-  if (_gst_check_fault_handler_is_setup)
-    return;
-
-  _gst_check_fault_handler_is_setup = TRUE;
-
-  memset (&action, 0, sizeof (action));
-  action.sa_handler = _gst_check_fault_handler_sighandler;
-
-  sigaction (SIGSEGV, &action, &oldaction_segv);
-  sigaction (SIGILL, &action, &oldaction_ill);
-  sigaction (SIGBUS, &action, &oldaction_bus);
-  sigaction (SIGABRT, &action, &oldaction_abrt);
-}
-#endif /* G_OS_UNIX && HAVE_EXECINFO_H */
-
 /* initialize GStreamer testing */
 void
 gst_check_init (int *argc, char **argv[])
@@ -192,10 +106,6 @@ gst_check_init (int *argc, char **argv[])
 
   check_cond = g_cond_new ();
   check_mutex = g_mutex_new ();
-
-#if defined(G_OS_UNIX) && defined(HAVE_EXECINFO_H)
-  _gst_check_fault_handler_setup ();
-#endif
 }
 
 /* message checking */
@@ -272,7 +182,7 @@ gst_check_setup_src_pad (GstElement * element,
 
 GstPad *
 gst_check_setup_src_pad_by_name (GstElement * element,
-    GstStaticPadTemplate * template, gchar * name)
+    GstStaticPadTemplate * template, const gchar * name)
 {
   GstPad *srcpad, *sinkpad;
 
@@ -297,7 +207,7 @@ gst_check_setup_src_pad_by_name (GstElement * element,
 }
 
 void
-gst_check_teardown_pad_by_name (GstElement * element, gchar * name)
+gst_check_teardown_pad_by_name (GstElement * element, const gchar * name)
 {
   GstPad *pad_peer, *pad_element;
 
@@ -306,23 +216,27 @@ gst_check_teardown_pad_by_name (GstElement * element, gchar * name)
   ASSERT_OBJECT_REFCOUNT (pad_element, "pad", 2);
   pad_peer = gst_pad_get_peer (pad_element);
 
-  if (gst_pad_get_direction (pad_element) == GST_PAD_SINK)
-    gst_pad_unlink (pad_peer, pad_element);
-  else
-    gst_pad_unlink (pad_element, pad_peer);
+  if (pad_peer) {
+    if (gst_pad_get_direction (pad_element) == GST_PAD_SINK)
+      gst_pad_unlink (pad_peer, pad_element);
+    else
+      gst_pad_unlink (pad_element, pad_peer);
 
-  /* caps could have been set, make sure they get unset */
-  gst_pad_set_caps (pad_peer, NULL);
+    /* caps could have been set, make sure they get unset */
+    gst_pad_set_caps (pad_peer, NULL);
+  }
 
   /* pad refs held by both creator and this function (through _get) */
   ASSERT_OBJECT_REFCOUNT (pad_element, "element pad_element", 2);
   gst_object_unref (pad_element);
   /* one more ref is held by element itself */
 
-  /* pad refs held by both creator and this function (through _get_peer) */
-  ASSERT_OBJECT_REFCOUNT (pad_peer, "check pad_peer", 2);
-  gst_object_unref (pad_peer);
-  gst_object_unref (pad_peer);
+  if (pad_peer) {
+    /* pad refs held by both creator and this function (through _get_peer) */
+    ASSERT_OBJECT_REFCOUNT (pad_peer, "check pad_peer", 2);
+    gst_object_unref (pad_peer);
+    gst_object_unref (pad_peer);
+  }
 }
 
 void
@@ -347,7 +261,7 @@ gst_check_setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
 
 GstPad *
 gst_check_setup_sink_pad_by_name (GstElement * element,
-    GstStaticPadTemplate * template, gchar * name)
+    GstStaticPadTemplate * template, const gchar * name)
 {
   GstPad *srcpad, *sinkpad;
 
@@ -422,8 +336,8 @@ gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2)
 }
 
 /**
- * gst_check_element_push_buffer:
- * @element: name of the element that needs to be created
+ * gst_check_element_push_buffer_list:
+ * @element_name: name of the element that needs to be created
  * @buffer_in: a list of buffers that needs to be puched to the element
  * @buffer_out: a list of buffers that we expect from the element
  * @last_flow_return: the last buffer push needs to give this GstFlowReturn
@@ -439,6 +353,7 @@ gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2)
  * 
  * Since: 0.10.18
  */
+/* FIXME 0.11: rename this function now that there's GstBufferList? */
 void
 gst_check_element_push_buffer_list (const gchar * element_name,
     GList * buffer_in, GList * buffer_out, GstFlowReturn last_flow_return)
@@ -528,6 +443,13 @@ gst_check_element_push_buffer_list (const gchar * element_name,
     GstBuffer *new = GST_BUFFER (buffers->data);
     GstBuffer *orig = GST_BUFFER (buffer_out->data);
 
+    GST_LOG ("orig buffer: size %u, caps %" GST_PTR_FORMAT,
+        GST_BUFFER_SIZE (orig), GST_BUFFER_CAPS (orig));
+    GST_LOG ("new  buffer: size %u, caps %" GST_PTR_FORMAT,
+        GST_BUFFER_SIZE (new), GST_BUFFER_CAPS (new));
+    GST_MEMDUMP ("orig buffer", GST_BUFFER_DATA (orig), GST_BUFFER_SIZE (orig));
+    GST_MEMDUMP ("new  buffer", GST_BUFFER_DATA (new), GST_BUFFER_SIZE (new));
+
     /* remove the buffers */
     buffers = g_list_remove (buffers, new);
     buffer_out = g_list_remove (buffer_out, orig);
@@ -571,9 +493,6 @@ gst_check_element_push_buffer (const gchar * element_name,
   out = g_list_append (out, buffer_out);
 
   gst_check_element_push_buffer_list (element_name, in, out, GST_FLOW_OK);
-
-  g_list_free (in);
-  g_list_free (out);
 }
 
 void
