@@ -136,8 +136,8 @@ GST_START_TEST (test_src_sink)
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -182,8 +182,8 @@ GST_START_TEST (test_livesrc_remove)
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -235,8 +235,8 @@ GST_START_TEST (test_livesrc_sink)
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -358,8 +358,8 @@ GST_START_TEST (test_livesrc2_sink)
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -427,8 +427,8 @@ GST_START_TEST (test_livesrc3_sink)
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -524,8 +524,8 @@ GST_START_TEST (test_unlinked_live)
   gst_bin_add (GST_BIN (pipeline), sink);
 
   /* link non live source to sink */
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -596,8 +596,8 @@ GST_START_TEST (test_delayed_async)
   fail_unless (pending == GST_STATE_VOID_PENDING, "have pending");
 
   /* link live source to sink */
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -655,8 +655,8 @@ GST_START_TEST (test_added_async)
   fail_unless (pending == GST_STATE_VOID_PENDING, "have pending");
 
   /* link live source to sink */
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -796,7 +796,7 @@ GST_START_TEST (test_add_live2)
 
   GST_DEBUG ("blocking srcpad");
   /* block source pad */
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   gst_pad_set_blocked_async (srcpad, TRUE, pad_blocked_cb, NULL);
 
   /* set source to PAUSED without adding it to the pipeline */
@@ -817,7 +817,7 @@ GST_START_TEST (test_add_live2)
   GST_DEBUG ("linking pads");
 
   /* link to sink */
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -863,8 +863,8 @@ GST_START_TEST (test_bin_live)
   gst_bin_add (GST_BIN (bin), sink);
   gst_bin_add (GST_BIN (pipeline), bin);
 
-  srcpad = gst_element_get_pad (src, "src");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  srcpad = gst_element_get_static_pad (src, "src");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_pad_link (srcpad, sinkpad);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
@@ -920,7 +920,7 @@ GST_START_TEST (test_fake_eos)
   sink = gst_element_factory_make ("fakesink", "sink");
   g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
 
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
 
   gst_bin_add (GST_BIN_CAST (pipeline), sink);
 
@@ -1008,6 +1008,9 @@ async_done_func (GstBus * bus, GstMessage * msg, GstElement * sink)
     fail_unless (position == 10 * GST_SECOND, "position is wrong");
   }
 
+  /* we must unref the message if we return DROP */
+  gst_message_unref (msg);
+
   /* we can drop the message, nothing is listening for it. */
   return GST_BUS_DROP;
 }
@@ -1016,7 +1019,7 @@ static void
 send_buffer (GstPad * sinkpad)
 {
   GstBuffer *buffer;
-  GstStateChangeReturn ret;
+  GstFlowReturn ret;
 
   /* push a second buffer */
   GST_DEBUG ("pushing last buffer");
@@ -1054,7 +1057,7 @@ GST_START_TEST (test_async_done)
   g_signal_connect (sink, "preroll-handoff", (GCallback) async_done_handoff,
       sink);
 
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
 
   ret = gst_element_set_state (sink, GST_STATE_PAUSED);
   fail_unless (ret == GST_STATE_CHANGE_ASYNC, "no ASYNC state return");
@@ -1076,8 +1079,8 @@ GST_START_TEST (test_async_done)
   format = GST_FORMAT_TIME;
   position = -1;
   qret = gst_element_query_position (sink, &format, &position);
-  fail_unless (qret == FALSE, "position wrong");
-  fail_unless (position == -1, "position is wrong");
+  fail_unless (qret == TRUE, "position wrong");
+  fail_unless (position == 10 * GST_SECOND, "position is wrong");
 
   /* Since we are paused and the preroll queue has a length of 2, this function
    * will return immediatly, the preroll handoff will be called and the stream
@@ -1167,6 +1170,8 @@ async_done_eos_func (GstBus * bus, GstMessage * msg, GstElement * sink)
 
     fail_unless (position == 10 * GST_SECOND, "position is wrong");
   }
+  /* we must unref the message if we return DROP */
+  gst_message_unref (msg);
   /* we can drop the message, nothing is listening for it. */
   return GST_BUS_DROP;
 }
@@ -1190,7 +1195,7 @@ GST_START_TEST (test_async_done_eos)
   g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
   g_object_set (G_OBJECT (sink), "preroll-queue-len", 1, NULL);
 
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
 
   ret = gst_element_set_state (sink, GST_STATE_PAUSED);
   fail_unless (ret == GST_STATE_CHANGE_ASYNC, "no ASYNC state return");
@@ -1212,8 +1217,8 @@ GST_START_TEST (test_async_done_eos)
   format = GST_FORMAT_TIME;
   position = -1;
   qret = gst_element_query_position (sink, &format, &position);
-  fail_unless (qret == FALSE, "position wrong");
-  fail_unless (position == -1, "position is wrong");
+  fail_unless (qret == TRUE, "position wrong");
+  fail_unless (position == 10 * GST_SECOND, "position is wrong");
 
   /* Since we are paused and the preroll queue has a length of 1, this function
    * will return immediatly. The EOS will complete the preroll and the  
@@ -1239,14 +1244,18 @@ GST_START_TEST (test_async_done_eos)
 GST_END_TEST;
 
 /* test: try changing state of sinks */
-Suite *
+static Suite *
 gst_sinks_suite (void)
 {
   Suite *s = suite_create ("Sinks");
   TCase *tc_chain = tcase_create ("general");
+  guint timeout = 10;
 
-  /* time out after 10s, not the default 3, we need this for the last test. */
-  tcase_set_timeout (tc_chain, 10);
+  /* time out after 10s, not the default 3, we need this for the last test.
+   * We need a longer timeout when running under valgrind though. */
+  if (g_getenv ("CK_DEFAULT_TIMEOUT") != NULL)
+    timeout = MAX (10, atoi (g_getenv ("CK_DEFAULT_TIMEOUT")));
+  tcase_set_timeout (tc_chain, timeout);
 
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_sink);
