@@ -171,7 +171,7 @@ print_event_masks (const GstEventMask * masks)
 }
 #endif
 
-static char *
+static const char *
 get_rank_name (gint rank)
 {
   switch (rank) {
@@ -942,7 +942,7 @@ print_children_info (GstElement * element)
 }
 
 static void
-print_blacklist ()
+print_blacklist (void)
 {
   GList *plugins, *cur;
   gint count = 0;
@@ -1491,7 +1491,7 @@ main (int argc, char *argv[])
 
   g_thread_init (NULL);
 
-  gst_tools_print_version ("gst-inspect");
+  gst_tools_set_prgname ("gst-inspect");
 
 #ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
@@ -1506,6 +1506,8 @@ main (int argc, char *argv[])
   gst_init (&argc, &argv);
 #endif
 
+  gst_tools_print_version ("gst-inspect");
+
   if (print_all && argc > 1) {
     g_print ("-a requires no extra arguments\n");
     return 1;
